/*
 *	This file contains IDA built-in function declarations
 *	and internal bit definitions.
 *	Each byte of the program has 32-bit flags
 *	(low 8 bits keep the byte value).
 *	These 32 bits are used in GetFlags/SetFlags functions.
 *	You may freely examine these bits using GetFlags()
 *	but I strongly discourage using SetFlags() function.
 *
 *	This file is subject to change without any notice.
 *	Future versions of IDA may use other definitions.
 */

#ifndef _IDC_IDC
#define _IDC_IDC

//--------------------------------------------------------------------------
#define	BADADDR		0xFFFFFFFFL	// Not allowed address value
#define	MAXADDR		0xFF000000L	// Max allowed address in IDA

//
//	Flag bit definitions (for GetFlags())
//
			
#define MS_VAL	0x000000FFL		// Mask for byte value
#define FF_IVL	0x00000100L		// Byte has value ?

#define hasValue(F)	((F & FF_IVL) != 0)	// any defined value?
#define byteValue(F)	(F & MS_VAL)	// quick replacement for Byte()

#define isLoaded(ea)	hasValue(GetFlags(ea))	// any defined value?

#define MS_CLS  0x00000600L		// Mask for typing
#define FF_CODE 0x00000600L		// Code ?
#define FF_DATA 0x00000400L		// Data ?
#define FF_TAIL 0x00000200L		// Tail ?
#define FF_UNK  0x00000000L		// Unknown ?

#define isCode(F)	((F & MS_CLS) == FF_CODE) // is code byte?
#define isData(F)	((F & MS_CLS) == FF_DATA) // is data byte?
#define isTail(F)	((F & MS_CLS) == FF_TAIL) // is tail byte?
#define isUnknown(F)	((F & MS_CLS) == FF_UNK)  // is unexplored byte?
#define isHead(F)	((F & FF_DATA) != 0)	  // is start of code/data?

//
//	Common bits
//

#define MS_COMM 0x000FF800L		// Mask of common bits
#define FF_COMM 0x00000800L		// Has comment?
#define FF_REF  0x00001000L		// has references?
#define FF_LINE 0x00002000L		// Has next or prev cmt lines ?
#define FF_NAME 0x00004000L		// Has user-defined name ?
#define FF_LABL 0x00008000L		// Has dummy name?
#define FF_FLOW 0x00010000L		// Exec flow from prev instruction?
#define FF_VAR  0x00080000L		// Is byte variable ?

#define isFlow	(F)	((F & FF_FLOW) != 0)
#define isVar	(F)	((F & FF_VAR ) != 0)
#define isExtra	(F)	((F & FF_LINE) != 0)
#define isRef	(F)	((F & FF_REF)  != 0)
#define hasName	(F)	((F & FF_NAME) != 0)

#define MS_0TYPE 0x00F00000L		// Mask for 1st arg typing
#define FF_0VOID 0x00000000L		// Void (unknown)?
#define FF_0NUMH 0x00100000L		// Hexadecimal number?
#define FF_0NUMD 0x00200000L		// Decimal number?
#define FF_0CHAR 0x00300000L		// Char ('x')?
#define FF_0SEG  0x00400000L		// Segment?
#define FF_0OFF  0x00500000L		// Offset?
#define FF_0NUMB 0x00600000L		// Binary number?
#define FF_0NUMO 0x00700000L		// Octal number?
#define FF_0ENUM 0x00800000L		// Enumeration?
#define FF_0FOP  0x00900000L		// Forced operand?

#define MS_1TYPE 0x0F000000L		// Mask for 2nd arg typing
#define FF_1VOID 0x00000000L		// Void (unknown)?
#define FF_1NUMH 0x01000000L		// Hexadecimal number?
#define FF_1NUMD 0x02000000L		// Decimal number?
#define FF_1CHAR 0x03000000L		// Char ('x')?
#define FF_1SEG  0x04000000L		// Segment?
#define FF_1OFF  0x05000000L		// Offset?
#define FF_1NUMB 0x06000000L		// Binary number?
#define FF_1NUMO 0x07000000L		// Octal number?
#define FF_1ENUM 0x08000000L		// Enumeration?
#define FF_1FOP  0x09000000L		// Forced operand?

#define isDefArg0(F)	((F & MS_0TYPE) != FF_0VOID)
#define isDefArg1(F)	((F & MS_1TYPE) != FF_1VOID)
#define isDec0(F)	((F & MS_0TYPE) == FF_0NUMD)
#define isDec1(F)	((F & MS_1TYPE) == FF_1NUMD)
#define isHex0(F)	((F & MS_0TYPE) == FF_0NUMH)
#define isHex1(F)	((F & MS_1TYPE) == FF_1NUMH)
#define isOct0(F)	((F & MS_0TYPE) == FF_0NUMO)
#define isOct1(F)	((F & MS_1TYPE) == FF_1NUMO)
#define isBin0(F)	((F & MS_0TYPE) == FF_0NUMB)
#define isBin1(F)	((F & MS_1TYPE) == FF_1NUMB)
#define isOff0(F)	((F & MS_0TYPE) == FF_0OFF)
#define isOff1(F)	((F & MS_1TYPE) == FF_1OFF)
#define isChar0(F)	((F & MS_0TYPE) == FF_0CHAR)
#define isChar1(F)	((F & MS_1TYPE) == FF_1CHAR)
#define isSeg0(F)	((F & MS_0TYPE) == FF_0SEG)
#define isSeg1(F)	((F & MS_1TYPE) == FF_1SEG)
#define isFop0(F)	((F & MS_0TYPE) == FF_0FOP)
#define isFop1(F)	((F & MS_1TYPE) == FF_1FOP)

//
//	Bits for DATA bytes
//

#define DT_TYPE 0xF0000000L		// Mask for DATA typing

#define FF_BYTE 0x00000000L		// byte
#define FF_WORD 0x10000000L		// word
#define FF_DWRD 0x20000000L		// dword
#define FF_QWRD 0x30000000L		// qword
#define FF_TBYT 0x40000000L		// tbyte
#define FF_ASCI 0x50000000L		// ASCII ?
#define FF_STRU 0x60000000L		// Struct ?
#define FF_XTRN 0x70000000L		// Extern data, unknown size
#define FF_FLOAT 0x80000000L		// float
#define FF_DOUBLE 0x90000000L		// double
#define FF_PACKREAL 0xA0000000L		// packed decimal real

//
//	Bits for CODE bytes
//

#define MS_CODE 0xF0000000L
#define FF_FUNC 0x10000000L		// function start?
#define FF_IMMD 0x40000000L		// Has Immediate value ?
#define FF_JUMP 0x80000000L		// Has jump table

#undef _notdefinedsymbol
#ifdef _notdefinedsymbol // There aren't declarations in IDC, so comment them

	List of built-in functions
	--------------------------

The following conventions are used in this list:
  'ea' is a linear address
  'success' is 0 if a function failed, 1 otherwise
  'void' means that function returns no meaningful value (always 0)

 All function parameter conversions are made automatically.


		      M I S C E L L A N E O U S


long	MK_FP		(long seg,long off);	// the same as [ seg, off ]
long	_lpoke		(long RAMea,long value); // poke a long integer into RAM
						// returns old value
long	_poke		(long RAMea,long value);// poke a byte into RAM
						// returns old value
long	_peek		(long RAMea);		// get a byte from RAM
char	form		(char format,...);      // works as sprintf
						// The resulting string should
						// be less than 255 characters.
char	substr		(char str,long x1,long x2); // substring [x1..x2-1]
						// if x2 == -1, then till end of line
long	strstr		(char str,char substr);	// find a substring, -1 - not found
long	strlen		(char str);		// calculate length
long	xtol		(char str);		// ascii hex -> number
						// (use long() for atol)
char	atoa		(long ea);		// returns address in
						// the form 'seg000:1234'
						// (the same as in line prefixes)
char	ltoa		(long n,long radix);	// convert to ascii string
long	atol		(char str);		// convert ascii decimal to long
success	AssignKey	(char action,long key);	// assign a key to an action
						// meaningful only before the main
						// menu is constructed
success	Jump		(long ea);		// move cursor to ea
						// screen is refreshed at the end
						// of IDC execution
void	Wait		();			// Wait for the end of
						// autoanalysis
void	Exit		(long code);		// Exit to DOS


      C H A N G E   P R O G R A M   R E P R E S E N T A T I O N


void	DeleteAll	();			// delete ALL information
						// about the program
long	MakeCode	(long ea);		// convert to instruction
						// returns number of bytes
						// occupied by the instruction
long	AnalyseArea	(long sEA,long eEA);	// analyse area and try to
						// convert to code all bytes
						// Returns 1-ok,0-CtrlBreak pressed
success	MakeName	(long ea,char name);	// assign a name to a location
success	MakeComm	(long ea,char comment);	// give a comment
success	MakeRptCmt	(long ea,char comment);	// give a repeatable comment
success	MakeArray	(long ea,long nitems);	// convert to an array
success	MakeStr		(long ea,long size);	// convert to ASCII string
success	MakeByte	(long ea);		// convert to byte
success	MakeWord	(long ea);		// convert to word
success	MakeDword	(long ea);		// convert to double-word
success	MakeQword	(long ea);		// convert to quadro-word
success	MakeFloat	(long ea);		// convert to float
success	MakeDouble	(long ea);		// convert to double
success	MakePackReal	(long ea);		// convert to packed real
success	MakeTbyte	(long ea);		// convert to 10 bytes (tbyte)
success	MakeStruct	(long ea,char strname);	// convert to structure instance
void	MakeUnkn	(long ea,long expand);	// convert to 'unknown'
						// expand!=0 => undefine consequent
						// instructions too
success	OpBinary	(long ea,int n);	// make operand binary
						// n=0 - first operand
						// n=1 - second, third etc. operands
						// n=-1 - all operands
success	OpOctal		(long ea,int n);	// make operand octal
						// n=0 - first operand
						// n=1 - second, third etc. operands
						// n=-1 - all operands
success	OpDecimal	(long ea,int n);	// make operand decimal
						// n=0 - first operand
						// n=1 - second, third etc. operands
						// n=-1 - all operands
success	OpHex		(long ea,int n);	// make operand hex
						// n=0 - first operand
						// n=1 - second, third etc. operands
						// n=-1 - all operands
success	OpChr		(long ea,int n);	// make operand character
						// n=0 - first operand
						// n=1 - second, third etc. operands
						// n=-1 - all operands
success	OpOff		(long ea,int n,long segbase); // make operand offset
						// if segbase==BADADDR, then operand isn't offset
						// n=0 - first operand
						// n=1 - second, third etc. operands
						// n=-1 - all operands
						// to define outer offset
						// combine 'n' with the
						// following bit:
#endif
#define OPND_OUTER	0x80			// outer offset base
#ifdef _notdefinedsymbol
success	OpSeg		(long ea,int n);	// make operand segment
						// n=0 - first operand
						// n=1 - second, third etc. operands
						// n=-1 - all operands
success	OpNumber	(long ea,int n);	// make operand number
						// n=0 - first operand
						// n=1 - second, third etc. operands
						// n=-1 - all operands
success	OpAlt		(long ea,long n,char str);// manually enter n-th operand
success	OpSign		(long ea,int n);	// change operand sign
void	MakeVar		(long ea);		// the location is 'variable'
void	ExtLinA		(long ea,long n,char line); // insert an additional line before the generated ones
void	ExtLinB		(long ea,long n,char line); // insert an additional line after the generated ones
void	DelExtLnA	(long ea,long n);	// delete an additional line before the generated ones
void	DelExtLnB	(long ea,long n);	// delete an additional line aftr  the generated ones
success	JmpTable	(long jmpea,long tableea,long nitems);	// define a jump table
void	PatchByte	(long ea,long value);	// change a byte
void	SetFlags	(long ea,long flags);	// change internal flags for ea
success	SetReg		(long ea,char reg,long value); // set value of segment register
void	AutoMark	(long ea,long type);	// plan address to analyse:
#endif

#define AU_UNK  10	// make unknown
#define AU_CODE 20	// convert to instruction
#define AU_PROC 30	// make function
#define AU_USED 40	// reanalyse

#ifdef _notdefinedsymbol


	       P R O D U C E   O U T P U T   F I L E S


void	WriteMap	(char file);		// produce a .map file
void	WriteTxt	(char file,long ea1,long ea2); // produce an .asm file
void	WriteExe	(char file);		// produce an executable file


		 C O M M O N   I N F O R M A T I O N

long	GetFlags	(long ea);		// get internal flags for ea
long	Byte		(long ea);		// get a byte at ea
long	Word		(long ea);		// get a word (2 bytes) at ea
long	Dword		(long ea);		// get a double-word (4 bytes) at ea
long	LocByName	(char name);		// BADADDR - no such name
long	SegByBase	(long base);		// BADADDR - no such segment
long	MinEA		();			// the minimal address defined in the program
long	MaxEA		();			// the maximal address defined in the program
long	BeginEA		();			// where execution starts
long	ScreenEA	();			// the current screen ea
long	SelStart	();			// the selected area start ea
						// BADADDR - no selected area
long	SelEnd		();			// the selected area end ea
						// BADADDR - no selected area
long	GetReg		(long ea,char reg);	// get segment register value
						// BADADDR - undefined or error
						// (selector, use AskSelector() to
						//  get its mapping)
#endif
//	for intel 80x86 processors:
#define R_ES  16
#define R_CS  17
#define R_SS  18
#define R_DS  19
#define R_FS  20
#define R_GS  21

#ifdef _notdefinedsymbol
long	NextAddr	(long ea);		// returns next defined address
						// BADADDR if no such address exists
long	PrevAddr	(long ea);		// returns prev defined address
						// BADADDR if no such address exists
long	NextHead	(long ea);		// returns next defined item address
						// BADADDR if no such address exists
long	PrevHead	(long ea);		// returns prev defined item address
						// BADADDR if no such address exists
long	NextNotTail	(long ea);		// returns next not tail address
						// BADADDR if no such address exists
long	PrevNotTail	(long ea);		// returns prev not tail address
						// BADADDR if no such address exists
long	ItemEnd		(long ea);		// returns address past end of
						// the item
long	ItemSize	(long ea);		// returns item size, min answer=1
char	Name		(long ea);		// get name of the byte
char	GetMnem		(long ea);		// get instruction name
char	GetOpnd		(long ea,long n);	// get instruction operand
						// n=0 - first operand
long	GetOpType	(long ea,long n);	// get operand type
						// n=0 - first operand
						// Returns:
						// -1	bad operand number passed
						// 0	None
						// 1	General Register (al,ax,es,ds...)
						// 2	Memory Reference
						// 3	Base + Index
						// 4	Base + Index + Displacement
						// 5	Immediate
						// 6	Immediate Far Address
						// 7	Immediate Near Address
						// 8	FPP register
						// 9	386 control register
						// 10	386 debug register
						// 11	386 trace register
						// 12	Condition (for Z80)
						// 13	bit (8051)
						// 14	bitnot (8051)
long	GetOperandValue	(long ea,long n);	// get instruction operand value
						// if unknown, returns -1
char	LineA		(long ea,long num);	// get additional line before generated ones
char	LineB		(long ea,long num);	// get additional line after generated ones
char	Comment		(long ea);		// get comment
char	RptCmt		(long ea);		// get repeatable comment
char	AltOp		(long ea,long n);	// get manually entered operand
//
//	The following functions search for the specified byte
//		ea - address to start from
//		flag |=1 - search forward
//		flag |=2 - search case-sensitive (only for FindText)
//	return BADADDR - not found
//
long	FindVoid	(long ea,long flag);
long	FindCode	(long ea,long flag);
long	FindData	(long ea,long flag);
long	FindProc	(long ea,long flag);
long	FindUnexplored	(long ea,long flag);
long	FindExplored	(long ea,long flag);
long	FindImmediate	(long ea,long flag,long value);
long	FindText	(long ea,long flag,long y,long x,char str);
		// y - number of text line at ea to start from (0..100)
		// x - x coordinate in this line
long	FindBinary	(long ea,long flag,char str);
		// str - a string as a user enters it for Search Text in Core
		//	example:  "0x41 0x42" - find 2 bytes 41h,42h


       G L O B A L   S E T T I N G S   M A N I P U L A T I O N


long	StringEnd	(long asciiendchar);	// set current ASCII end character
						// returns old value
long	StringStp	(long asciistopchar);	// set current ASCII break character
						// returns old value
long	LowVoids	(long lowlimit);	// set current low limit for voids
						// returns old value
long	HighVoids	(long highlimit);	// set current high limit for voids
						// returns old value
long	TailDepth	(long taildepth);	// set current tail depth
						// returns old value
long	Direction	(long direction);	// set current search direction
						// returns old value
long	Analysis	(long analysis);	// enable/disable auto analysis
						// returns old value
long	Tabs		(long tabulations);	// enable/disable tabulations in output file
						// returns old value
long	Comments	(long comments);	// enable/disable automatic comments
						// returns old value
long	Voids		(long voids);		// enable/disable void marks display
						// returns old value
long	XrefShow	(long xrefshow);	// set number of cross-references to display
						// returns old value
long	Indent		(long indent);		// set indention for instruntions
						// returns old value
long	CmtIndent	(long cmtindent);	// set indention for comments
						// returns old value
long	AutoShow	(long autoshow);	// enable/disable autoanalysis display
						// returns old value
success	SetPrcsr	(char processor);	// set processor type


	  I N T E R A C T I O N   W I T H   T H E   U S E R


char	AskStr		(char defval,char prompt); // ask a string
long	AskAddr		(long defval,char prompt); // BADADDR - no or bad input
long	AskSeg		(long defval,char prompt); // BADADDR - no or bad input
char	AskIdent	(char defval,char prompt);
void	Message		(char format,...);	// show a message in messages window
void	Warning		(char format,...);	// show a warning a dialog box
void	Fatal		(char format,...);	// exit IDA immediately


		       S E G M E N T A T I O N


***********************************************
** get a selector value
	arguments:	sel - the selector
	returns:	selector value if found
			otherwise the input value (sel)
	note:		selector values are always in paragraphs

long	AskSelector	(long sel);	// returns paragraph

***********************************************
** find a selector which has the specifed value
	arguments:	val - value to search for
	returns:	selector if found
			otherwise the input value (val)
	note:		selector values are always in paragraphs

long	FindSelector	(long val);

***********************************************
** set a selector value
	arguments:	sel - the selector, should be
			less than 0xFFFF
			val - new value of selector
	returns:	nothing
	note:		ida supports up to 64 selectors.
			if 'sel' == 'val' then the
			selector is destroyed because
			it has no importance

void	SetSelector	(long sel,long value);

***********************************************
** delete a selector
	arguments:	sel - the selector to delete
	returns:	nothing
	note:		if the selector is found, it will
			be deleted

void	DelSelector	(long sel);

***********************************************
** SEGMENT FUNCTIONS

long	FirstSeg	();			// returns start of the first
						// segment, BADADDR - no segments
long	NextSeg		(long ea);		// returns start of the next
						// segment, BADADDR - no more segs
long	SegStart	(long ea);		// returns start of the segment
						// BADADDR if bad address passed
long	SegEnd		(long ea);		// return end of the segment
						// this address doesn't belong
						// to the segment
						// BADADDR if bad address passed
char	SegName		(long ea);		// returns name of the segment
						// "" if bad address passed
success	SegCreate	(long startea,long endea,long base,long use32,long alignment,long combination);
success	SegDelete	(long segea,long disable);	// disable=1: exclude all bytes of the segment
							// from the disassembled text.
success	SegBounds	(long segea,long startea,long endea);
success	SegRename	(long segea,char name);
success	SegClass	(long segea,char class);
success	SegAlign	(long segea,long alignment);
#endif
	#define saAbs	   0	// Absolute segment.
	#define saRelByte  1	// Relocatable, byte aligned.
	#define saRelWord  2	// Relocatable, word (2-byte, 16-bit) aligned.
	#define saRelPara  3	// Relocatable, paragraph (16-byte) aligned.
	#define saRelPage  4	// Relocatable, aligned on 256-byte boundary (a "page"
	                	// in the original Intel specification).
	#define saRelDble  5	// Relocatable, aligned on a double word (4-byte)
	                	// boundary. This value is used by the PharLap OMF for
        	        	// the same alignment.
	#define saRel4K    6	// This value is used by the PharLap OMF for page (4K)
	                	// alignment. It is not supported by LINK.
#ifdef _notdefinedsymbol
success	SegComb		(long segea,long combination);
#endif
	#define scPriv     0    // Private. Do not combine with any other program
                        	// segment.
	#define scPub      2    // Public. Combine by appending at an offset that meets
        	        	// the alignment requirement.
	#define scPub2     4    // As defined by Microsoft, same as C=2 (public).
	#define scStack    5    // Stack. Combine as for C=2. This combine type forces
	                	// byte alignment.
	#define scCommon   6    // Common. Combine by overlay using maximum size.
	#define scPub3     7    // As defined by Microsoft, same as C=2 (public).
#ifdef _notdefinedsymbol
success	SegAddrng	(long segea,long addrng);
long	SegByName	(char segname);		// returns segment base
success	SegDefReg	(long segea,char reg,long value);
						// set default value for segment reg
***********************************************
** set segment type
	arguments:	segea - any address within segment
			type  - new segment type:
#endif
#define SEG_NORM	0
#define SEG_XTRN	1	// segment with 'extern' definitions
#define SEG_CODE	2	// pure code segment
#define SEG_DATA	3	// pure data segment
#define SEG_IMP		4	// impdef segment
#define SEG_GRP		6	// group of segments
#define SEG_NULL	7	// zero-length segment
#define SEG_UNDF	8	// undefined segment type
#ifdef _notdefinedsymbol
	returns:	!=0 - ok

success	SetSegmentType	(long segea,long type);



		   C R O S S   R E F E R E N C E S


//	Flow types:
#endif
#define fl_CF	16		// Call Far
#define fl_CN	17		// Call Near
#define fl_JF	18		// Jump	Far
#define fl_JN	19		// Jump	Near
#define fl_US	20		// User specified
#define fl_F	21		// Ordinary flow
#ifdef _notdefinedsymbol
					// Mark exec flow 'from' 'to'
void	Rflow	(long From,long To,long flowtype);
long	Ruflow	(long From,long To,int undef);// Unmark exec flow 'from' 'to'
					// undef - make 'To' undefined if no
					//	  more references to it
					// returns 1 - planned to be
					// made undefined

// The following functions include the ordinary flows:
long	Rfirst	(long From);		// Get first xref from 'From'
long	Rnext	(long From,long current);// Get next xref from
long	RfirstB	(long To);		// Get first xref to 'To'
long	RnextB	(long To,long current);	// Get next xref to 'To'

// The following functions don't take into account the ordinary flows:
long	Rfirst0	(long From);
long	Rnext0	(long From,long current);
long	RfirstB0(long To);
long	RnextB0	(long To,long current);

//	Data reference types:
#endif
#define dr_O	1			// Offset
#define dr_W	2			// Write
#define	dr_R	3			// Read
#define	dr_T	4			// Text (names in manual operands)
#ifdef _notdefinedsymbol

void	setDref	(long From,long To,long drefType);	// Mark Data Ref
void	delDref	(long From,long To);	// Unmark Data Ref

long	Dfirst	(long From);		// Get first refered address
long	Dnext	(long From,long current);
long	DfirstB	(long To);		// Get first referee address
long	DnextB	(long To,long current);

long	XrefType(void);			// returns type of the last xref
					// obtained by [RD]first/next[B0]
					// functions. Return values
					// are fl_... or dr_...

			   F I L E   I / O


***********************************************
** open a file
	arguments: similiar to C fopen()
	returns:	0 -error
			otherwise a file handle

long	fopen		(char file,char mode);

***********************************************
** close a file
	arguments:	file handle
	returns:	nothing

void	fclose		(long handle);

***********************************************
** get file length
	arguments:	file handle
	returns:	-1 - error
			otherwise file length in bytes

long	filelength	(long handle);

***********************************************
** set cursor position in the file
	arguments:	handle	- file handle
			offset	- offset from origin
			origin	- 0 = from start of file
				  1 = from current cursor position
				  2 = from end of file
	returns:	0 - ok
			otherwise error

long	fseek		(long handle,long offset,long origin);

***********************************************
** get cursor position in the file
	arguments:	file handle
	returns:	-1 - error
			otherwise current cursor position

long	ftell		(long handle);

***********************************************
** load file into IDA database
	arguments:	handle	- file handle
			pos	- position in the file
			ea	- linear address to load
			size	- number of bytes to load
	returns:	0 - error
			1 - ok

success	loadfile	(long handle,long pos,long ea,long size);

***********************************************
** save from IDA database to file
	arguments:	handle	- file handle
			pos	- position in the file
			ea	- linear address to save from
			size	- number of bytes to save
	returns:	0 - error
			1 - ok

success	savefile	(long handle,long pos,long ea,long size);

***********************************************
** read one byte from file
	arguments:	handle	- file handle
	returns:	-1 - error
			otherwise a byte read.

long	fgetc		(long handle);

***********************************************
** write one byte to file
	arguments:	handle	- file handle
			byte	- byte to write
	returns:	0 - ok
			-1 - error

long	fputc		(long byte,long handle);

***********************************************
** fprintf
	arguments:	handle	- file handle
			format  - format string
	returns:	0 - ok
			-1 - error

long	fprintf		(long handle,char format,...);

***********************************************
** read 2 bytes from file
	arguments:	handle	- file hanlde
			mostfirst 0 - least significant byte is first (intel)
				  1 - most  significant byte is first
	returns:	-1 - error
			otherwise: a 16-bit value

long	readshort	(long handle,long mostfirst);

***********************************************
** read 4 bytes from file
	arguments:	handle	- file hanlde
			mostfirst 0 - least significant byte is first (intel)
				  1 - most  significant byte is first
	returns:	a 32-bit value

long	readlong	(long handle,long mostfirst);

***********************************************
** write 2 bytes to file
	arguments:	handle	- file hanlde
			word	- a 16-bit value to write
			mostfirst 0 - least significant byte is first (intel)
				  1 - most  significant byte is first
	returns:	0 - ok

long	writeshort	(long handle,long word,long mostfirst);

***********************************************
** write 4 bytes to file
	arguments:	handle	- file hanlde
			dword	- a 32-bit value to write
			mostfirst 0 - least significant byte is first (intel)
				  1 - most  significant byte is first
	returns:	0 - ok

long	writelong	(long handle,long dword,long mostfirst);

***********************************************
** read a string from file
	arguments:	handle	- file hanlde
	returns:	a string
			on EOF, returns -1

char	readstr		(long handle);

***********************************************
** write a string to file
	arguments:	handle	- file hanlde
			str	- string to write
	returns:	0 - ok

long	writestr	(long handle,char str);


			  F U N C T I O N S


***********************************************
** create a function
	arguments:	start,end - function bounds
	returns:	!=0 - ok

success	MakeFunction(long start,long end);

***********************************************
** delete a function
	arguments:	ea - any address belonging to the function
	returns:	!=0 - ok

success DelFunction(long ea);

***********************************************
** change function end address
	arguments:	ea - any address belonging to the function
			end - new function end address
	returns:	!=0 - ok

success SetFunctionEnd(long ea,long end);

***********************************************
** find next function
	arguments:	ea - any address belonging to the function
	returns:	-1 - no more functions
			otherwise returns the next function start address

long NextFunction(long ea);

***********************************************
** find previous function
	arguments:	ea - any address belonging to the function
	returns:	-1 - no more functions
			otherwise returns the previous function start address

long PrevFunction(long ea)

***********************************************
** retrieve function flags
	arguments:	ea - any address belonging to the function
	returns:	-1 - function doesn't exist
			otherwise returns the flags:
#endif
#define FUNC_NORET	0x00000001L	// function doesn't return
#define FUNC_FAR	0x00000002L	// far function
#ifdef _notdefinedsymbol

long GetFunctionFlags(long ea);

***********************************************
** change function flags
	arguments:	ea - any address belonging to the function
	returns:	!=0 - ok

success SetFunctionFlags(long ea,long flags);

***********************************************
** retrieve function name
	arguments:	ea - any address belonging to the function
	returns:	null string - function doesn't exist
			otherwise returns function name

char GetFunctionName(long ea);

***********************************************
** ask the user to select a function
	arguments:	title - title of the dialog box
	returns:	-1 - user refused to select a function
			otherwise returns the selected function start address

long ChooseFunction(char title);

***********************************************
** convert address to 'funcname+offset' string
	arguments:	ea - address to convert
	returns:	if the address belongs to a function then
			  return a string formed as 'name+offset'
			  where 'name' is a function name
			  'offset' is offset within the function
			else
			  return null string

char GetFuncOffset(long ea);


		       E N T R Y   P O I N T S


***********************************************
** retrieve number of entry points
	arguments:	none
	returns:	number of entry points

long GetEntryPointQty(void);

***********************************************
** add entry point
	arguments:	ordinal  - entry point number
				   if entry point doesn't have an ordinal
				   number, 'ordinal' should be equal to 'ea'
			ea       - address of the entry point
			name     - name of the entry point. If null string,
			           the entry point won't be renamed.
			makecode - if 1 then this entry point is a start
				   of a function. Otherwise it denotes data
				   bytes.
	returns:	0 - entry point with the specifed ordinal already
				exists
			1 - ok

success AddEntryPoint(long ordinal,long ea,char name,long makecode);

***********************************************
** retrieve entry point ordinal number
	arguments:	index - 0..GetEntryPointQty()-1
	returns:	0 if entry point doesn't exist
			otherwise entry point ordinal

long GetEntryOrdinal(long index);

***********************************************
** retrieve entry point address
	arguments:	ordinal - entry point number
				  it is returned by GetEntryPointOrdinal()
	returns:	-1 if entry point doesn't exist
			otherwise entry point address.
			If entry point address is equal to its ordinal
			number, then the entry point has no ordinal.

long GetEntryPoint(long ordinal);

***********************************************
** rename entry point
	arguments:	ordinal - entry point number
			name    - new name
	returns:	!=0 - ok

success RenameEntryPoint(long ordinal,char name);



			     F I X U P S


***********************************************
** find next address with fixup information
	arguments:	ea - current address
	returns:	-1 - no more fixups
			otherwise returns the next address with
						fixup information

long GetNextFixupEA(long ea);

***********************************************
** find previous address with fixup information
	arguments:	ea - current address
	returns:	-1 - no more fixups
			otherwise returns the previous address with
						fixup information

long GetPrevFixupEA(long ea);

***********************************************
** get fixup target type
	arguments:	ea - address to get information about
	returns:	-1 - no fixup at the specified address
			otherwise returns fixup target type:
#endif
#define	FIXUP_MASK	0xF	// mask for fixup types:
#define	FIXUP_BYTE	0	//   Low-order byte (8-bit displacement or
				//   low byte of 16-bit offset).
#define FIXUP_OFF16	1	//   16-bit offset.
#define FIXUP_SEG16	2	//   16-bit base--logical segment base
				//   (selector).
#define FIXUP_PTR32	3	//   32-bit long pointer (16-bit base:16-bit
				//   offset).
#define FIXUP_OFF32	4	//   32-bit offset.
#define FIXUP_PTR48	5	//   48-bit pointer (16-bit base:32-bit offset)
#define FIXUP_HI8	6	//   high 8 bits of 16bit offset
#define FIXUP_SELFREL	0x10	// self-relative?
#define FIXUP_EXTDEF	0x20	// target is a location (otherwise - segment)
#define FIXUP_UNUSED	0x40	// fixup is ignored by IDA
#ifdef _notdefinedsymbol

long GetFixupTgtType(long ea);

***********************************************
** get fixup target selector
	arguments:	ea - address to get information about
	returns:	-1 - no fixup at the specified address
			otherwise returns fixup target selector

long GetFixupTgtSel(long ea);

***********************************************
** get fixup target offset
	arguments:	ea - address to get information about
	returns:	-1 - no fixup at the specified address
			otherwise returns fixup target offset

long GetFixupTgtOff(long ea);

***********************************************
** get fixup target displacement
	arguments:	ea - address to get information about
	returns:	-1 - no fixup at the specified address
			otherwise returns fixup target displacement

long GetFixupTgtDispl(long ea);

***********************************************
** set fixup information
	arguments:	ea        - address to set fixup information about
			type      - fixup type. see GetFixupTgtType()
				    for possible fixup types.
			targetsel - target selector
			targetoff - target offset
			displ     - displacement
	returns:	none

void SetFixup(long ea,long type,long targetsel,long targetoff,long displ);

***********************************************
** delete fixup information
	arguments:	ea - address to delete fixup information about
	returns:	none

void DelFixup(long ea);


		   M A R K E D   P O S I T I O N S


***********************************************
** mark position
	arguments:	ea      - address to mark
			lnnum   - number of generated line for the 'ea'
			x       - x coordinate of cursor
			y       - y coordinate of cursor
			slot    - slot number: 1..20
			          if the specifed value is not within the
			          range, IDA will ask the user to select slot.
			comment - description of the mark.
				  Should be not empty.
	returns:	none

void MarkPosition(long ea,long lnnum,long x,long y,long slot,char comment);

***********************************************
** get marked position
	arguments:	slot    - slot number: 1..20
			          if the specifed value is <= 0
			          range, IDA will ask the user to select slot.
	returns:	-1 - the slot doesn't contain a marked address
			otherwise returns the marked address

long GetMarkedPos(long slot);

***********************************************
** get marked position comment
	arguments:	slot    - slot number: 1..20
	returns:	null string if the slot doesn't contain
					a marked address
			otherwise returns the marked address comment

char GetMarkComment(long slot);


			 S T R U C T U R E S


***********************************************
** get number of defined structure types
	arguments:	none
	returns:	number of structure types

long GetStrucQty(void);

***********************************************
** get index of first structure type
	arguments:	none
	returns:	-1 if no structure type is defined
			index of first structure type.
			Each structure type has an index and ID.
			INDEX determines position of structure definition
			 in the list of structure definitions. Index 1
			 is listed first, after index 2 and so on.
			 The index of a structure type can be changed any 
			 time, leading to movement of the structure definition
			 in the list of structure definitions.
			ID uniquely denotes a structure type. A structure
			 gets a unique ID at the creation time and this ID
			 can't be changed. Even when the structure type gets
			 deleted, its ID won't be resued in the future.

long GetFirstStrucIdx(void);

***********************************************
** get index of last structure type
	arguments:	none
	returns:	-1 if no structure type is defined
			index of last structure type.
			See GetFirstStrucIdx() for the explanation of
			structure indices and IDs.

long GetLastStrucIdx(void);

***********************************************
** get index of next structure type
	arguments:	current structure index
	returns:	-1 if no (more) structure type is defined
			index of the next structure type.
			See GetFirstStrucIdx() for the explanation of
			structure indices and IDs.

long GetNextStrucIdx(long index);

***********************************************
** get index of previous structure type
	arguments:	current structure index
	returns:	-1 if no (more) structure type is defined
			index of the presiouvs structure type.
			See GetFirstStrucIdx() for the explanation of
			structure indices and IDs.

long GetPrevStrucIdx(long index);

***********************************************
** get structure index by structure ID
	arguments:	structure ID
	returns:	-1 if bad structure ID is passed
			otherwise returns structure index.
			See GetFirstStrucIdx() for the explanation of
			structure indices and IDs.

long GetStrucIdx(long id);

***********************************************
** get structure ID by structure index
	arguments:	structure index
	returns:	-1 if bad structure index is passed
			otherwise returns structure ID.
			See GetFirstStrucIdx() for the explanation of
			structure indices and IDs.

long GetStrucId(long index);

***********************************************
** get structure ID by structure name
	arguments:	structure type name
	returns:	-1 if bad structure type name is passed
			otherwise returns structure ID.

long GetStrucIdByName(char name);


***********************************************
** get structure type name
	arguments:	structure type ID
	returns:	-1 if bad structure type ID is passed
			otherwise returns structure type name.

char GetStrucName(long id);

***********************************************
** get structure type comment
	arguments:	id         - structure type ID
			repeatable - 1: get repeatable comment
				     0: get regular comment
	returns:	null string if bad structure type ID is passed
			otherwise returns comment.

char GetStrucComment(long id,long repeatable);

***********************************************
** get size of a structure
	arguments:	id         - structure type ID
	returns:	-1 if bad structure type ID is passed
			otherwise returns size of structure in bytes.

long GetStrucSize(long id);

***********************************************
** get number of members of a structure
	arguments:	id         - structure type ID
	returns:	-1 if bad structure type ID is passed
			otherwise returns number of members.

long GetMemberQty(long id);

***********************************************
** get previous offset in a structure
	arguments:	id     - structure type ID
			offset - current offset
	returns:	-1 if bad structure type ID is passed
			   or no (more) offsets in the structure
			otherwise returns previous offset in a structure.
			NOTE: IDA allows 'holes' between members of a
			      structure. It treats these 'holes'
			      as unnamed arrays of bytes.
			This function returns a member offset or a hole offset.
			It will return size of the structure if input
			'offset' is bigger than the structure size.

long GetStrucPrevOff(long id,long offset);

***********************************************
** get next offset in a structure
	arguments:	id     - structure type ID
			offset - current offset
	returns:	-1 if bad structure type ID is passed
			   or no (more) offsets in the structure
			otherwise returns next offset in a structure.
			NOTE: IDA allows 'holes' between members of a
			      structure. It treats these 'holes'
			      as unnamed arrays of bytes.
			This function returns a member offset or a hole offset.
			It will return size of the structure if input
			'offset' belongs to the last member of the structure.

long GetStrucNextOff(long id,long offset);

***********************************************
** get offset of the first member of a structure
	arguments:	id            - structure type ID
	returns:	-1 if bad structure type ID is passed
	 		   or structure has no members
			otherwise returns offset of the first member.
			NOTE: IDA allows 'holes' between members of a
			      structure. It treats these 'holes'
			      as unnamed arrays of bytes.

long GetFirstMember(long id);

***********************************************
** get offset of the last member of a structure
	arguments:	id            - structure type ID
	returns:	-1 if bad structure type ID is passed
	 		   or structure has no members
			otherwise returns offset of the last member.
			NOTE: IDA allows 'holes' between members of a
			      structure. It treats these 'holes'
			      as unnamed arrays of bytes.

long GetLastMember(long id);

***********************************************
** get offset of a member of a structure by the member name
	arguments:	id            - structure type ID
			member_name   - name of structure member
	returns:	-1 if bad structure type ID is passed
			   or no such member in the structure
			otherwise returns offset of the specified member.

long GetMemberOffset(long id,char member_name);

***********************************************
** get name of a member of a structure
	arguments:	id            - structure type ID
			member_offset - member offset. The offset can be
					any offset in the member. For example,
					is a member is 4 bytes long and starts
					at offset 2, then 2,3,4,5 denote
					the same structure member.
	returns:	-1 if bad structure type ID is passed
			   or no such member in the structure
			otherwise returns name of the specified member.

char GetMemberName(long id,long member_offset);

***********************************************
** get comment of a member
	arguments:	id            - structure type ID
			member_offset - member offset. The offset can be
					any offset in the member. For example,
					is a member is 4 bytes long and starts
					at offset 2, then 2,3,4,5 denote
					the same structure member.
			repeatable - 1: get repeatable comment
				     0: get regular comment
	returns:	null string if bad structure type ID is passed
			   or no such member in the structure
			otherwise returns comment of the specified member.

char GetMemberComment(long id,long member_offset,long repeatable);

***********************************************
** get size of a member
	arguments:	id            - structure type ID
			member_offset - member offset. The offset can be
					any offset in the member. For example,
					is a member is 4 bytes long and starts
					at offset 2, then 2,3,4,5 denote
					the same structure member.
	returns:	-1 if bad structure type ID is passed
			   or no such member in the structure
			otherwise returns size of the specified
				  	  member in bytes.

long GetMemberSize(long id,long member_offset);

***********************************************
** get type of a member
	arguments:	id            - structure type ID
			member_offset - member offset. The offset can be
					any offset in the member. For example,
					is a member is 4 bytes long and starts
					at offset 2, then 2,3,4,5 denote
					the same structure member.
	returns:	-1 if bad structure type ID is passed
			   or no such member in the structure
			otherwise returns type of the member, see bit
			definitions above. If the member type is a structure
			then function GetMemberStrid() should be used to
			get the structure type id.

long GetMemberFlag(long id,long member_offset);

***********************************************
** get structure id of a member
	arguments:	id            - structure type ID
			member_offset - member offset. The offset can be
					any offset in the member. For example,
					is a member is 4 bytes long and starts
					at offset 2, then 2,3,4,5 denote
					the same structure member.
	returns:	-1 if bad structure type ID is passed
			   or no such member in the structure
			otherwise returns structure id of the member.
			If the current member is not a structure, returns -1.

long GetMemberStrId(long id,long member_offset);

***********************************************
** define a new structure type
	arguments:	index         - index of new structure type
			If another structure has the specified index,
			then index of that structure and all other
			structures will be increentedfreeing the specifed
			index. If index is == -1, then the biggest index
			number will be used.
			See GetFirstStrucIdx() for the explanation of
			structure indices and IDs.

			name - name of the new structure type.

	returns:	-1 if can't define structure type because of
			bad structure name: the name is ill-formed or is
			already used in the program.
			otherwise returns ID of the new structure type

long AddStruc(long index,char name);

***********************************************
** delete a structure type
	arguments:	id            - structure type ID
	returns:	0 if bad structure type ID is passed
			1 otherwise the structure type is deleted. All data
			and other structure types referencing to the
			deleted structure type will be displayed as array
			of bytes.

success DelStruc(long id);

***********************************************
** change structure index
	arguments:	id      - structure type ID
			index	- new index of the structure
			See GetFirstStrucIdx() for the explanation of
			structure indices and IDs.
	returns:	!=0 - ok

long SetStrucIdx(long id,long index);

***********************************************
** change structure name
	arguments:	id      - structure type ID
			name	- new name of the structure
	returns:	!=0 - ok

long SetStrucName(long id,char name);

***********************************************
** change structure comment
	arguments:	id      - structure type ID
			comment	- new comment of the structure
			repeatable - 1: change repeatable comment
				     0: change regular comment
	returns:	!=0 - ok

long SetStrucComment(long id,char comment,long repeatable);

***********************************************
** add structure member
	arguments:	id      - structure type ID
			name    - name of the new member
			offset  - offset of the new member
			flag    - type of the new member. Should be one of
				  FF_BYTE..FF_PACKREAL (see above).
			typeid  - structure id if 'flag' == FF_STRU
				  Denotes type of the member is the member
				  itself is a structure. Otherwise should be
				  -1.
				  if isOff0(flag) then typeid specifies
				  the offset base.
			nitems  - number of items in the new member
	returns:	0 - ok, otherwise error code:
#endif
#define	STRUC_ERROR_MEMBER_NAME	  1	// already have member with this name (bad name)
#define	STRUC_ERROR_MEMBER_OFFSET 2	// already have member at this offset
#define	STRUC_ERROR_MEMBER_SIZE   3	// bad number of items or bad sizeof(type)
#ifdef _notdefinedsymbol

long AddStrucMember(long id,char name,long offset,long flag,
 		    long typeid,long nitems);

***********************************************
** delete structure member
	arguments:	id            - structure type ID
			member_offset - offset of the member
	returns:	!=0 - ok.
			NOTE: IDA allows 'holes' between members of a
			      structure. It treats these 'holes'
			      as unnamed arrays of bytes.

long DelStrucMember(long id,long member_offset);

***********************************************
** change structure member name
	arguments:	id            - structure type ID
			member_offset - offset of the member
			name	      - new name of the member
	returns:	!=0 - ok.

long SetMemberName(long id,long member_offset,char name);

***********************************************
** change structure member type
	arguments:	id            - structure type ID
			member_offset - offset of the member
			flag    - new type of the member. Should be one of
				  FF_BYTE..FF_PACKREAL (see above).
			typeid  - structure id if 'flag' == FF_STRU
				  Denotes type of the member is the member
				  itself is a structure. Otherwise should be
				  -1.
				  if isOff0(flag) then typeid specifies
				  the offset base.
			nitems  - number of items in the member
	returns:	!=0 - ok.

long SetMemberType(long id,long member_offset,long flag,long typeid,long nitems);

***********************************************
** change structure member comment
	arguments:	id      - structure type ID
			member_offset - offset of the member
			comment	- new comment of the structure member
			repeatable - 1: change repeatable comment
				     0: change regular comment
	returns:	!=0 - ok

long SetMemberComment(long id,long member_offset,char comment,long repeatable);


#endif // _notdefinedsymbol
//--------------------------------------------------------------------------
// Compatibility macros:

#define	OpOffset(ea,base)	OpOff(ea,-1,base)
#define	OpNum(ea)		OpNumber(ea,-1)
#define	OpChar(ea)		OpChr(ea,-1)
#define	OpSegment(ea)		OpSeg(ea,-1)
#define	OpDec(ea)		OpDecimal(ea,-1)
#define	MakeProc(ea,n)		// nothing to do
#define	MakeEndp(ea,n)		// nothing to do
#define	OpAlt1(ea,str)		OpAlt(ea,0,str)
#define	OpAlt2(ea,str)		OpAlt(ea,1,str)

#endif // _IDC_IDC
